package exercises.ex_z4;

import java.util.List;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;


import static exercises.ex_z4.AuctionState.*;

public class Auctioneer implements Runnable {
    private String name;
    private Queue<Bid> bids = new ConcurrentLinkedQueue<>();
    private Bid maximumBid;
    private Thread countdownThread = null;
    private boolean isAuctionActive = false;
    private AuctionState auctionState = PRESENTING_ITEM;
    private final List<Item> items;


    public Auctioneer(String name, List<Item> items) {
        this.name = name;
        this.items = items;
    }

    private void announce(String msg) {
        System.out.println("[Auctioneer] " + msg);
    }

    public void presentItem() throws InterruptedException {
        Item item = items.getFirst();
        announce("Now selling this new item: '" + item.name() + "'");
        announce("Get ready to bid...");
        Thread.sleep(2000);
        announce("Bidding open !");
        auctionState = BIDDING;
    }

    public void placeBid(Bid bid) {
        if (auctionState == BIDDING) {
            System.out.println(bid.bidder() + " bids $" + bid.amount());
            bids.add(bid);
        }
        notify();
    }

    private void countdown() {
        try {
            Thread.sleep(1000);
            System.out.println("Three");
            Thread.sleep(1000);
            System.out.println("Two");
            Thread.sleep(1000);
            System.out.println("One");
            Thread.sleep(1000);
            auctionState = SOLD;
            notify();
        } catch (InterruptedException _) {}
    }

    public void startCountdown() {
        countdownThread = new Thread(this::countdown);
        countdownThread.start();
    }

    public Bid getMaximumBid() {
        return maximumBid;
    }

    public AuctionState getAuctionState() {
        return auctionState;
    }

    public void finalizeBidding() throws InterruptedException {
        announce("And sold for $" + maximumBid.amount() + " !");
        Item item = items.removeFirst();
        announce(maximumBid.bidder() + " is now the proud owner of " + item);
        maximumBid.bidder().addItem(item);
        Thread.sleep(2000);
        if (items.isEmpty()) {
            auctionState = FINISHED;
        } else {
            auctionState = PRESENTING_ITEM;
        }
    }

    @Override
    public void run() {
        while (auctionState != FINISHED) {
            try {
                switch (auctionState) {
                    case PRESENTING_ITEM -> {
                        presentItem();
                    }
                    case BIDDING -> {
                        Bid current_bid = bids.poll();
                        if (current_bid == null) {
                            startCountdown();
                            wait();
                        } else {
                            if (current_bid.amount() > maximumBid.amount()) {
                                maximumBid = current_bid;
                                announce("New best bid : " + maximumBid.amount() + " by " + maximumBid.bidder());
                            }
                        }
                    }
                    case SOLD -> {
                        finalizeBidding();
                    }
                }

            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
        announce("The auction is now finished ! Thank you for participating !");
    }
}
